@@ -3,6 +3,8 @@
 # ----------------------------------------------------------------------------------------------------
 import mlflow
 import pickle
+import requests
+from os import getenv
 from pathlib import Path
 from mlflow.exceptions import RestException, MlflowException
 from shutil import rmtree
@@ -34,21 +36,21 @@ def load_model_mlflow(model_name: str, artifacts_destination_path: str = "temp_a
         Path.mkdir(caminho_artefatos, parents=True, exist_ok=True)
     except PermissionError:
         msg = f"Não foi possível criar a pasta de destino dos artefatos '{artifacts_destination_path}'. Permissão " \
-              f"de escrita negada. Programa abortado!"
+              f"de escrita negada."
         LOGGER.error(msg)
         raise PermissionError(msg) from None
 
-    stage = 'Production'
+    alias = 'production'
 
     # Carrega o modelo que está em produção
     try:
-        modelo = mlflow.pyfunc.load_model(model_uri=f"models:/{model_name}/{stage}")
+        modelo = mlflow.pyfunc.load_model(model_uri=f"models:/{model_name}@{alias}")
     except RestException:
-        msg = f"O modelo '{model_name}' no estágio '{stage}' não foi encontrado. Programa abortado!"
+        msg = f"O modelo '{model_name}' com o alias '{alias}' não foi encontrado."
         LOGGER.error(msg)
         raise RuntimeError(msg) from None
     except MlflowException as e:
-        msg = f"Não foi possível carregar o modelo '{model_name}'. Mensagem do MLFlow: '{e}'. Programa abortado!"
+        msg = f"Não foi possível carregar o modelo '{model_name}'. Mensagem do MLFlow: '{e}'."
         LOGGER.error(msg)
         raise RuntimeError(msg) from None
 
@@ -59,7 +61,7 @@ def load_model_mlflow(model_name: str, artifacts_destination_path: str = "temp_a
         mlflow.artifacts.download_artifacts(artifact_uri=endereco_base_artefatos, dst_path=str(caminho_artefatos))
     except MlflowException as e:
         msg = f"Não foi possível carregar os artefatos no endereço '{endereco_base_artefatos}'. " \
-              f"Mensagem do MLFlow: '{e}'. Programa abortado!"
+              f"Mensagem do MLFlow: '{e}'."
         LOGGER.error(msg)
         raise RuntimeError(msg) from None
 
@@ -100,7 +102,7 @@ def load_production_params_mlflow(model_name: str) -> dict:
     else:
         msg += f"Não foi possível carregar os parâmetros de produção do modelo '{model_name}'. Certifique-se que o " \
                f"modelo exista e que possua os parâmetros persistidos num dicionário, através do Pickle, com o nome " \
-               f"'TrainingParams.pkl'. Programa abortado!"
+               f"'TrainingParams.pkl'."
         LOGGER.error(msg)
         raise RuntimeError(msg)
 
@@ -140,7 +142,7 @@ def load_production_datasets_names_mlflow(model_name: str) -> dict:
     else:
         msg += f"Não foi possível carregar os nomes dos datasets de produção do modelo '{model_name}'. Certifique-se " \
                f"que o modelo exista e que possua os nomes dos datasets persistidos num dicionário, através do " \
-               f"Pickle, com o nome 'TrainingDatasetsNames.pkl'. Programa abortado!"
+               f"Pickle, com o nome 'TrainingDatasetsNames.pkl'."
         LOGGER.error(msg)
         raise RuntimeError(msg)
 
@@ -180,6 +182,66 @@ def load_production_baseline_mlflow(model_name: str) -> dict:
     else:
         msg += f"Não foi possível carregar o baseline de produção do modelo '{model_name}'. Certifique-se que o " \
                f"modelo exista e que possua o baseline persistido num dicionário, através do Pickle, com o nome " \
-               f"'BaselineMetrics.pkl'. Programa abortado!"
+               f"'BaselineMetrics.pkl'."
         LOGGER.error(msg)
         raise RuntimeError(msg)
+
+
+def get_models_versions_mlflow(models_names: list) -> dict:
+    """
+    Obtém as versões de alguns modelos que estão sendo providos pelo provider.
+        :param models_names: Lista com os nomes dos modelos para obtenção das versões.
+        :return: Dicionário com o nome de cada modelo como chave e a respectiva versão como valor.
+    """
+    # Obtém as credenciais para acesso ao MLflow
+    mlflow_uri = getenv("MLFLOW_TRACKING_URI")
+    if not mlflow_uri:
+        msg = "Não foi possível obter a variável de ambiente 'MLFLOW_TRACKING_URI'"
+        LOGGER.error(msg)
+        raise RuntimeError(msg)
+
+    mlflow_user = getenv("MLFLOW_TRACKING_USERNAME")
+    if not mlflow_user:
+        msg = "Não foi possível obter a variável de ambiente 'MLFLOW_TRACKING_USERNAME'"
+        LOGGER.error(msg)
+        raise RuntimeError(msg)
+
+    mlflow_password = getenv("MLFLOW_TRACKING_PASSWORD")
+    if not mlflow_password:
+        msg = "Não foi possível obter a variável de ambiente 'MLFLOW_TRACKING_PASSWORD'"
+        LOGGER.error(msg)
+        raise RuntimeError(msg)
+
+    models_versions = {}
+
+    # Faz a busca da versão dos modelos utilizando alias
+    for model_name in models_names:
+        params = {'name': model_name, 'alias': "production"}
+
+        try:
+            response = requests.get(
+                f"{mlflow_uri}/api/2.0/mlflow/registered-models/alias",
+                params=params,
+                auth=(mlflow_user, mlflow_password)
+            )
+        except BaseException as e:
+            msg = f"Não foi possível obter as versões para os modelos: {models_names}. Erro reportado pelo MLflow: " \
+                  f"{e}."
+            LOGGER.error(msg)
+            raise RuntimeError(e) from None
+
+        dados_modelo = response.json()
+
+        if "model_version" not in dados_modelo:
+            msg = f"A resposta do MLflow não contém a chave 'model_version'. Resposta do MLflow: {dados_modelo}"
+            LOGGER.error(msg)
+            raise KeyError(msg)
+
+        if "version" not in dados_modelo['model_version']:
+            msg = f"A resposta do MLflow não contém a chave 'version'. Resposta do MLflow: {dados_modelo}"
+            LOGGER.error(msg)
+            raise KeyError(msg)
+
+        models_versions[model_name] = dados_modelo['model_version']['version']
+
+    return models_versions
